#!/bin/bash


#This shell script calls a variety of homemade perl script in order to sort out wVulC genes using in silico criteria 

#Inputs:
#a directory called "genomes" containing:
#- a file named "Orthologygroups.txt" that is the ouput of OrthoMCL. It must groups all ortholog groups from Wolbachia genome of interest.
#- the protein file in fasta format of wCon ("proteines_wCon.fas") and wVulC.
#a directory called "programmes" containing all perl script programme
#a "fasta.pm" file that contains some homemade perl libraries.

#Initialization
#Creation of directories
#Directory of non candidate
mkdir un
#Intermediate directory
mkdir intermediaires
#Directory of final candidate genes
mkdir CAN

#####################
###PART 1 (Manual)###
#####################

###Filtration of repeats, pseudogenes, small cds and smaller chunk of cds cut in two.

#Remove all repeats : remove all ORF overlapping "repeats regions", all genes annotated "transposase", all genes annotated "reverse transcriptase", all genes annotated "mobile intron", all genes overlapping phage region and all genes annotated "phage". (one can call "sansME.pl" to remove genes annotated "phage", "transposase" and "reverse transcriptase").
#Remove all pseudogenes (gene annotated "fragments")
#Remove all small cds (length <50aa, one can call filtretaille.pl)
#When there is one cds cut in two pieces, remove only the smaller chunk

#Create a file containing only the list of remaining genes in "intermediaires/wVulC_sansME3_sanspseudo.list"



############
###PART 2###
############

###Filter Wolbachia Arthopod core genome###

#Using the orthogroup file of OrthoMCL, get the core genome of Wolbachia infecting arthropod
grep "wMel|" genomes/Orthologygroups.txt | grep "wPipPel|" | grep "wRi|"| grep "wNo|" | grep "wHa|" > genomes/corearthropod.txt
#Extract gene candidate list that do not belong to Wolbachia arthopod core genome.
programmes/extract_candidats.pl intermediaires/wVulC_sansME3_sanspseudo.list genomes/corearthropod.txt
mv wVulC.core un/
mv wVulC_sanscore.list intermediaires/
#Extract sequence of remaining candidate genes
./programmes/extractseq.pl intermediaires/wVulC_sanscore.list intermediaires/wVulC_sansME3_sanspseudo.faa	
mv candidates.faa wVulC_candidates.faa


############
###PART 3###
############


###BLAST with wCon###
#Create the database
mkdir databases_blast
formatdb -i genomes/proteines_wCon.fas -p T -n databases_blast/proteines_wCon
#Call the blast
./programmes/blastprot.pl -q wVulC_candidates.faa -blast databases_blast/proteines_wCon
#Clean the directory
rm *.blast
rm formatdb.log
mkdir blasthits
mv hits blasthits/
mv manyhits blasthits/
mv nohits blasthits/
mv identiques un/
mv candidats intermediaires/		
#Create with the last EN FAIRE UN .list et un .faa à mettre dans CAN
./programmes/extractseq.pl candidates.list intermediaires/wVulC_sansME3_sanspseudo.faa 







