#!/usr/bin/perl	-W

#Step 0: Check Bioperl and Blast software are installed.
#Step 1: get 3 files with fasta format :
#	- genome file
#	- cds from genomic file
#	- protein file
#Step 2: Check fasta library is located at the root directory
#Step 3: 
#	- enter file names to the appopriate location (in part Initialisation)
#	- Check Blast parameters
#	- Execute Blast
#Step 4: Execute the program
#Step 5: read output.


use strict;

use fasta;
use Bio::Tools::Run::StandAloneBlast;
use Bio::SeqIO;
print "#########################\n";
print "###PROGRAM BLASTGENOME###\n";
print "#########################\n\n";



###########################################
###########INITIALISATION##################
###########################################

print "***INITIALISATION***\n\n";

my $key;				
my $value;

#Initialisation of blast parameters and arguments
my $genome1_file;
my $evalue= 0.000001;			#minimal E-value minimale
my $program = 'blastp';		#blast type
my $database;
my $coveragemin = 00; 			#minimal querycoverage
my $identitymin = 70;

for my $args (0 .. $#ARGV){
 SWITCH: {

    # Blast query
    if( $ARGV[$args] =~ /^-q/)
      { $args++; $genome1_file = $ARGV[$args]; last SWITCH; }

      # blast database
    if( $ARGV[$args] =~ /^-blast/)
      { $args++; $database = $ARGV[$args]; last SWITCH; }
	
    # evalue
    if( $ARGV[$args] =~ /^-e/)
      { $args++; $evalue = $ARGV[$args]; last SWITCH; }

    # MINIMAL SIZE OF SEQUENCE
    if( $ARGV[$args] =~ /^-l/)
      { $args++; $coveragemin = $ARGV[$args]; last SWITCH; }

    # MINIMAL SIMILARITY
    if( $ARGV[$args] =~ /^-s/)
      { $args++; $identitymin = $ARGV[$args]; last SWITCH; }

    if( $ARGV[$args] =~ /^-program/)
      { $args++; $program = $ARGV[$args]; last SWITCH; }
 
  }

}

print "evalue = $evalue\t coverage = $coveragemin\t id = $identitymin\t q=$genome1_file\tblast=$database\n\n";

(my $blastfile) = ($genome1_file=~/(\w*)\_/);
$blastfile .=".blast";


print "Lecture des fichiers ";
#Extract query fasta file
my %genome1 = extractSequenceFasta(getFileData($genome1_file));
print "\tFAIT\n";


#Blast parameters
#F option to F is to remove low complexity filter
#r=2 is the bonus when there is a hit.
my @params = (	-program => $program, 
		-database => $database,
		-output => $blastfile,
		F =>'F',
		e => $evalue,
		r =>2,
		);

print "\tFAIT\n\n";





###########################################
##############BLAST########################
###########################################

print "***BLAST***\n\n";
my $factory = Bio::Tools::Run::StandAloneBlast -> new(@params);
my $blastoutput;

#Output blast parameters
my $hitname;		
my $lengthquery;	
my $lengthseq;		
my $querycoverage;	
my $hitstart;		
my $hitend;		
my $querystring;
my $hitstring;
my $identity;
my $querystart;
my $queryend;
my $gaps;
my $conserved;
my $identical;
my $positives;
my $querydifference;
my $lengthseqhit;

my @hit;
my @zerohit;
my @manyhit;
my @compteurhit;

my @uncandidates;
my @candidates;

#Value initialization
my $i = 0;
my $percent=0;
$evalue=1;
my $nbrgenes=scalar keys %genome1;
#Blast, read all different entry from the query file
while($key = each %genome1){
	
	my $blast_query = Bio::Seq -> new (-seq => $genome1{$key});
	my $blast_report= $factory -> blastall($blast_query);
	
	#print the percent of computing on screen
	$percent =100*$i/$nbrgenes;
	printf("\r %.2f%%",$percent);		
	
	my $blast_results = $blast_report->next_result();
	if(my $hit = $blast_results->next_hit)
	{
		#get blast value		
		while(my $hsp = $hit->next_hsp)
		{
			$evalue= $hsp -> evalue;
			$hitname= $hit -> name;
			$hitstart = $hsp -> start('hit');
			$hitend = $hsp -> end('hit');
			$querystart = $hsp -> start('query');
			$queryend = $hsp -> end('query');
			$gaps = $hsp -> gaps;
			$conserved = $hsp ->  num_conserved;	#=positives
			$identical = $hsp ->  num_identical;
			$lengthquery= $hsp -> length('query');
			$lengthseq= $blast_results -> query_length;
			$lengthseqhit= $hit -> length;
			$querycoverage=100*$lengthquery/$lengthseq;
			$querydifference= abs($lengthseqhit-$lengthseq);
			$identity = $hsp -> frac_identical;
			$identity = 100*$identity;
			$positives = $hsp -> frac_conserved;
			$positives = 100*$positives;

				
				my $datas = $key."\t".$querycoverage."\t".$identity."\t".$positives."\t".$hitstart."\t".$hitend."\t".$gaps."\t".$conserved."\t".$identical."\t".$hitname;

				my $string1 = "@hit";
				my $string2 = "@manyhit";

				if (($string1 =~ /$key/) && ($string2 !~ /$key/))
				{	

					push(@manyhit,$key);
				}
				push (@hit,$datas);
				
				if($positives==100 && $querydifference<=10)
				{
					push(@uncandidates,$datas);

				}
				else
				{
					push(@candidates,$datas);

				}
				
			last;
		}
	}
	else{push(@zerohit,$key); push(@candidates,$key); 


					}


	$evalue=1;
	$i++;


}
printf ("\r%.2f%%\tFAIT\n\n",$percent);
$" ="\n";
my $nbno_hits = $#zerohit+1;
my $nb_hits = $i - $nbno_hits;
print "\nSur $i genes testes, $nbno_hits n'ont pas eu de hits et $nb_hits ont eu au moins un hit.\n\n";

print "candidats: $#candidates+1\tuncandidats: $#uncandidates+1\n";

open (CANDIDATS, ">candidats") or die "Cannot open file candidats\n";
print CANDIDATS "queryname\tquerycoverage\tidentity\tpositives\thitstart\thitend\tgaps\tconserved\tidentical\thitname\n";
print CANDIDATS "@candidates ";
close (CANDIDATS);

open (UNCANDIDATS, ">identiques") or die "Cannot open file identiques\n";
print UNCANDIDATS "queryname\tquerycoverage\tidentity\tpositives\thitstart\thitend\tgaps\tconserved\tidentical\thitname\n";
print UNCANDIDATS "@uncandidates ";
close (UNCANDIDATS);


open (NOHITS, ">nohits") or die "Cannot open file nohits\n";
open (MANYHITS, ">manyhits") or die "Cannot open file manyhits\n";
open (HITS,">hits") or die "Cannot open file hits\n";
print NOHITS "@zerohit ";
print HITS "@hit ";
print MANYHITS "@manyhit ";
close(NOHITS);
close (HITS);
close(MANYHITS);




