#Functions to read a fasta file and extract a sequence.
#Contains the functions below:

#	getFileData('FILE_NAME')
#		Input: File Name
#		Output: Data of the file in a Table

#	extractSequenceFasta(@TAB)
#		Input: a table with fasta format
#		Output: names and sequences in dictionnary format.

#	printSequence($sequence,$length)
#		Input: Sequence and length of a line in a fasta file
#			taille d'une ligne du fichier FASTA
#		Output: Print fasta file without header






sub getFileData{
	my($filename)=@_;
	my (@filedata)=();
	
	unless( open(GET_FILE_DATA, $filename)) {
		print STDERR "Cannot open file \"$filename\"\n\n";
	}
	
	@filedata= <GET_FILE_DATA>;
	close GET_FILE_DATA;

	return @filedata;
}

sub extractSequenceFasta {
	my(@fastaFileData)=@_;
	my @sequence = ();
	my @nom = ();
	my $nb=-1;
	my %ADN;

	foreach my $line (@fastaFileData) {
		#Discard empty lines
		if ($line =~ /^\s*$/){next;}
		#Discard comment lines
		elsif ($line =~ /^\s*#$/){next;}
		#Keep header
		elsif ($line =~ />/){

			$line=~s/>//g;
			$line=~s/\n//;
			$line=~s/\//\-/g;
			push (@nom,$line);
			$nb++;	

		}
		#Keep sequence lines
		else {
			$line=~s/\s//g;
			$sequence[$nb].=$line;
		}
	}
	@ADN{@nom}=@sequence;
	return %ADN;
}

sub printSequence {
	my ($sequence, $length)=@_;
	for (my $pos = 0 ; $pos < length($sequence) ; $pos += $length) {
		print substr ($sequence, $pos, $length),"\n";
	}
}

1;
